#!/usr/bin/tclsh

# 2D точки придставляются как списки { x y }

proc pstring { p } {  ;# форматирование 2D точки
   return [ format "\[%.2f,%.2f\]" [ lindex $p 0 ] [ lindex $p 1 ] ]
}

proc minus { f t } {  ;# замыкание 2-х векторов
   set dx [ expr [ lindex $f 0 ] - [ lindex $t 0 ] ]
   set dy [ expr [ lindex $f 1 ] - [ lindex $t 1 ] ]
   return [ list $dx $dy ]
}

proc pabs { p } {     ;# длина вектора
   set x2 [ tcl::mathfunc::pow [ lindex $p 0 ] 2 ]
   set y2 [ tcl::mathfunc::pow [ lindex $p 1 ] 2 ]
   return [ tcl::mathfunc::sqrt [ expr $x2 + $y2 ] ]
}

proc polar { p } {    ;# полярное представление вектора
   return [ list [ pabs $p ] [ tcl::mathfunc::atan2 [ lindex $p 1 ] [ lindex $p 0 ] ] ]
}

proc perimeter { lp { i 0 } } {
   set n [ expr [ llength $lp ] - 1 ]
   set next [ expr { $i < $n } ? [ expr $i + 1 ] : 0 ] 
   set dist [ pabs [ minus [ lindex $lp $i ] [ lindex $lp $next ] ] ] 
   if { $i < $n } {
      return [ expr $dist + [ perimeter $lp $next ] ]
   } else {
      return $dist
   } 
}

proc square { lp } {
   set n [ expr [ llength $lp ] - 2 ]
   set summa 0.0
   for { set i 1 } { $i <= $n } { } {
      set p1 [ polar [ minus [ lindex $lp $i ] [ lindex $lp 0 ] ] ]
      incr i
      set p2 [ polar [ minus [ lindex $lp $i ] [ lindex $lp 0 ] ] ]
      set ampl [ expr ( [ lindex $p1 0 ] * [ lindex $p2 0 ] / 2 ) ]
      set asin [ tcl::mathfunc::sin [ tcl::mathfunc::abs [ expr [ lindex $p1 1 ] - [ lindex $p2 1 ] ] ] ]
      set summa [ expr $summa + $ampl * $asin ] 
   }
   return $summa
}

set pattern {^[\s\t]*[-+]?(\d+|\.\d+|\d+\.\d*)\s+(\d+|\.\d+|\d+\.\d*)\s*$}
while { true } {
   set polygon [ list ]
   puts "координаты вершин в формате: X Y (^D конец ввода)"
   set number 1
   while { true } {
      puts -nonewline "вершина № $number : "
      flush stdout
      set res [ gets stdin line ]
      if { $res <= 0 } {
         if { $res == -1 } { puts -nonewline "\r" }
         break
      }
      set result [ regexp $pattern $line match x y ]
      if { $result != 1 } {
         puts "ошибка ввода"
         continue
      }
      incr number 
      set point [ list $x $y ]
      lappend polygon $point ;# многоугольник: {{} {} ... {}}
   }  
   set number [ llength $polygon ]
   if { $number <= 2 } break
   set msg "вершин $number : "
   foreach p $polygon {
      set msg [ concat $msg [ pstring $p ] " " ]
   }
   puts "$msg"
   puts "периметр = [ format {%.4f} [ perimeter $polygon ] ]"
   puts "площадь = [ format {%.4f} [ square $polygon ] ]"
}
